;====================================================;
; PS2MouseSetUp.                                     ;
;====================================================;
PS2MouseSetUp:
	push  eax
	push  ebx
	cli
	mov   bl,0xa8
	call  KeyboardCommand
	call  KeyboardRead
	mov   bl,0x20
	call  KeyboardCommand
	call  KeyboardRead
	or    al,3
	mov   bl,0x60
	push  eax
	call  KeyboardCommand
	pop   eax
	call  KeyboardWrite
	mov   bl,0xd4
	call  KeyboardCommand
	mov   al,0f4h
	call  KeyboardWrite
	call  KeyboardRead
	mov   bl,0a7h				
	call  KeyboardCommand
	mov   bl,0a8h
	call  KeyboardCommand
	sti
	pop   ebx
	pop   eax
	ret

;====================================================;
; KeyboardRead.                                      ;
;====================================================;
KeyboardRead:
	push  ecx
	push  edx
	mov   ecx,0xffff
KeyReadLoop:
	in    al,0x64
	test  al,1
	jnz   KeyReadReady
	loop  KeyReadLoop
	mov   ah,1
	jmp   KeyReadExit
KeyReadReady:
	push  ecx
	mov   ecx,32
KeyReadDelay:
	loop  KeyReadDelay
	pop   ecx
	in    al,0x60
	xor   ah,ah
KeyReadExit:
	pop   edx
	pop   ecx
	ret

;====================================================;
; keyboardWrite.                                     ;
;====================================================;
KeyboardWrite:
	push  ecx
	push  edx
	mov   dl,al
	mov   ecx,0ffffh
KbdWaitLoop1:
	in    al,0x64
	test  al,0x20
	jz    KbdWaitOk1
	loop  KbdWaitLoop1
	mov   ah,1
	jmp   KbdWaitExit
KbdWaitOk1:
	in    al,0x60
	mov   ecx,0xffff
KbdWaitLoop:
	in    al,0x64
	test  al,2
	jz    KbdWriteOk
	loop  KbdWaitLoop
	mov   ah,1
	jmp   KbdWaitExit
KbdWriteOk:
	mov   al,dl
	out   0x60,al
	mov   ecx,0ffffh
KbdWaitLoop3:
	in    al,64h
	test  al,2
	jz    KbdWriteOk3
	loop  KbdWaitLoop3
	mov   ah,1
	jmp   KbdWaitExit
KbdWriteOk3:
	mov   ah,8
KbdWaitLoop4:
	mov   ecx,0xffff
KbdWaitLoop5:
	in    al,64h
	test  al,1
	jnz   KbdWriteOk4
	loop  KbdWaitLoop5
	dec   ah
	jnz   KbdWaitLoop4
KbdWriteOk4:
	xor   ah,ah
KbdWaitExit:
	pop   edx
	pop   ecx
	ret

;====================================================;
; KeyboardCommand.                                   ;
;====================================================;
KeyboardCommand:
	xor   eax,eax
	mov   ecx,0xffff
CommandWait:
	in    al,64h
	test  al,2
	jz    CommandSend
	loop  CommandWait
	jmp   CommandError
CommandSend:
	mov   al,bl
	out   64h,al
	mov   ecx,0xffff
CommandAccept:
	in    al,0x64
	test  al,2
	jz    CommandOk
	loop  CommandAccept
CommandError:
	mov   ah,1
	jmp   CommandExit
CommandOk:
	xor   ah,ah
CommandExit:
	ret

 ;----------------------------------------------------;
 ; Mouse ISR                                          ;
 ;----------------------------------------------------;
Mouse_isr:
	pusha
	push  es
	push  cs
	pop   ds
	mov   ax,0xb800
	mov   es,ax
	mov   byte[es:0000],' ' 
	mov   bl,0xad
	call  KeyboardCommand
	xor   di,di
	mov   ecx,3
PS2MouseISR_ReadPacket: 
	push  ecx
	call  KeyboardRead
	pop   ecx		
	or    ah,ah
	jnz   PS2MouseISR_ExitReadPacket
	mov   byte[ds:buffer+di],al
	inc   di
	loop  PS2MouseISR_ReadPacket
PS2MouseISR_ExitReadPacket:
	mov   ax,di
	mov   al,[ds:buffer]	
	and   al,7h
	mov   ah,al
	and   ah,1
	shl   ah,2
	mov   bl,al
	and   bl,6
	shr   bl,1
	or    ah,bl
	mov   al,[buttons]
	mov   [buttons],ah
	xor   ah,al
	shl   ah,1
	or    [eventmsk],ah	
	mov   al,[ds:buffer+1]
	mov   ah,[ds:buffer]
	and   ah,10h
	shr   ah,4
	neg   ah
	add   [current_x],ax	
	or    ax,ax
	setnz bl
	or    [eventmsk],bl
	mov   al,[ds:buffer+2]
	mov   ah,[ds:buffer]
	and   ah,0x20
	shr   ah,5
	neg   ah
	neg   ax
	add   [current_y], ax	
	or    ax,ax
	setnz bl
	or    [eventmsk],bl
 ;----------------------------------------------------;
 ; check boundaries                                   ;
 ;----------------------------------------------------;
	mov   dx,[current_x]
	cmp   dx,[min_x]
	jnl   PS2MouseISR_CheckMaxX
	mov   dx,[min_x]
	jmp   PS2MouseISR_StoreNewX
PS2MouseISR_CheckMaxX:
	cmp   dx,[max_x]
	jl    PS2MouseISR_StoreNewX
	mov   dx,[max_x]
PS2MouseISR_StoreNewX:
	mov   [current_x],dx
	mov   dx,[current_y]
	cmp   dx,[min_y]
	jnl   PS2MouseISR_CheckMaxY
	mov   dx,[min_y]
	jmp   PS2MouseISR_StoreNewY
PS2MouseISR_CheckMaxY:
	cmp   dx,[max_y]
	jl    PS2MouseISR_StoreNewY
	mov   dx,[max_y]
PS2MouseISR_StoreNewY:
	mov   [current_y],dx
	call  PutMousePointer
@@:
	xor   ax,ax
	mov   al,[buttons]
	cmp   al,0
	je    NoButton
	mov   ax,0xb800
	mov   es,ax
	mov   byte[es:0000],'B' 
NoButton:   
	mov   bl,0aeh
	call  KeyboardCommand
	mov   al, 0x20
	out   0xa0, al
	out   0x20,al
	pop es
	popa
	iret

 ;----------------------------------------------------;
 ; PutMousePointer                                    ;
 ;----------------------------------------------------;
PutMousePointer:
	pusha				       
	push  es
	push  ds
	mov    ax,0xb800
	mov    es,ax
	xor   di,di
	call  GetMousePointer1
	mov   ax,[SaveChar]
	mov   [es:di],ax      
	xor   di,di
	call  GetMousePointer2
	mov   ax,word[es:di]
	mov   [SaveChar],ax
	not   ah
	xor   ah,0x80
	mov   [es:di],ax		    
	mov   ax,[current_x]
	mov   [current_x1],ax
	mov   ax,[current_y]
	mov   [current_y1],ax
	pop   ds
	pop   es
	popa
	ret


 ;----------------------------------------------------;
 ; Gets mouse pointer                                 ;
 ;----------------------------------------------------;

GetMousePointer2:				      
	push  eax					   
	push  ebx
	xor   eax,eax
	xor   ebx,ebx
	mov   bx,[current_x]
	shl   ebx,1
	mov   di, bx
	xor   ebx,ebx
	mov   bx,[current_y]
	mov   eax,0xa0
	mul   bx
	add   di,ax
	pop   ebx
	pop   eax
	ret

GetMousePointer1:				      
	push  eax					   
	push  ebx
	xor   eax,eax
	xor   ebx,ebx
	mov   bx,[current_x1]
	shl   ebx,1
	mov   di, bx
	xor   ebx,ebx
	mov   bx,[current_y1]
	mov   eax,0xa0
	mul   bx
	add   di,ax
	pop   ebx
	pop   eax
	ret

 ;----------------------------------------------------;
 ; SetMouseOn                                         ;
 ;----------------------------------------------------;
SetMouseOn:
	cli
	mov   [min_x],0 
	mov   [max_x],79
	mov   [min_y],0
	mov   [max_y],24
	mov   [buttons],0
	mov   [current_x],40
	mov   [current_y],12
	mov   [current_x1],40
	mov   [current_y1],12
	mov   bl,0a8h
	call  KeyboardCommand
	sti
	ret

 ;----------------------------------------------------;
 ; SetMouseOff                                        ;
 ;----------------------------------------------------;
SetMouseOff:
	cli
	mov   bl,0a7h
	call  KeyboardCommand
	sti
	ret

;====================================================;
; Data.                                              ;
;====================================================;
MouseShow	db	0
SaveChar	dw	0
min_x		dw	0
max_x		dw	79
min_y		dw	0 
max_y		dw	24
eventmsk	db	0
buffer	db	0,0,0,0,0,0,0,0
buttons 	db	0
current_x	dw	40 
current_y	dw	12
current_x1	dw	40  
current_y1	dw	12


